@@ -73,6 +73,10 @@ import traceback
 from types import MethodType
 from typing import (Any, Callable, ContextManager, Dict, Iterable, List,
                     NamedTuple, no_type_check, Optional, Tuple, TypeVar, Union)
+try:
+    from typing import Protocol
+except ImportError:
+    from typing_extensions import Protocol  # type: ignore[misc]
 
 import gevent.local
 from gevent.server import StreamServer
@@ -101,6 +105,11 @@ _T = TypeVar('_T')
 _FN = TypeVar('_FN', bound=Callable)
 
 
+class _GETTER(Protocol):
+    def __call__(self, for_write: bool) -> Any: ...
+    def setter(self, fset: _RSET) -> _RSET: ...
+
+
 class RegisterType:
     """Base class for implementing read/write functions for registers
 
@@ -301,12 +310,13 @@ class _Options(NamedTuple):
 
 
 # Used solely for type-checking
-class _Server:
+class _Server(Protocol):
     route_map: Any
     route: Callable[[Any, List[int], List[int], List[int]], Callable[[_FN], _FN]]
     _default_options: _Options
-    register: MethodType
-    update_defaults: MethodType
+    def register(self, address: Union[int, Iterable[int]], kind: RegisterType, *,
+                 slave_id: int = None, byteorder: str = None, mixed: bool = None) -> 'Register': ...
+    def update_defaults(self, slave_id: int = None, byteorder: str = None, mixed: bool = None) -> None: ...
 
 _S = TypeVar('_S', bound=_Server)
 
@@ -317,8 +327,8 @@ def patch_server(server: _S, slave_id: int = 1, byteorder: str = 'big', mixed: b
     _validate_slave_id(slave_id)
     _validate_byteorder(byteorder)
     server._default_options = _Options(slave_id, byteorder, mixed)
-    server.register = MethodType(Register, server)
-    server.update_defaults = MethodType(update_defaults, server)
+    server.register = MethodType(Register, server)  # type: ignore[assignment]
+    server.update_defaults = MethodType(update_defaults, server)  # type: ignore[assignment]
     return server
 
 
@@ -430,7 +440,7 @@ class Register:
         self.write_word(offset, self.get_write_value, self.set_value, value)
         _local.writes[self] = _local.values[self]
 
-    def __call__(self, fget: _RGET, *, depth: int = 1) -> _RGET:
+    def __call__(self, fget: _RGET, *, depth: int = 1) -> _GETTER:
         """Decorate a getter function
 
         The getter should accept a single boolean argument, which
@@ -466,7 +476,7 @@ class Register:
             return fset
 
         fget.setter = setter  # type: ignore[attr-defined]
-        return fget
+        return fget  # type: ignore[return-value]
 
     def attribute(self, obj: Any, name: str, writable: Union[bool, Callable] = True) -> None:
         """Use an object attribute to store a register value
